import 'package:dartz/dartz.dart';

extension BirdEitherExtension<L, R> on Either<L, R> {
  Option<L> toLeftOption() => fold(some, (_) => none());
}

Option<List<A>> optionOfList<A>(List<A> i) => (i.isEmpty) ? none() : some(i);

Option<Iterable<A>> optionOfIterable<A>(Iterable<A> i) => (i.isEmpty) ? none() : some(i);

extension BirdTupleExtension<A> on Option<A> {
  TwoOption<A, B> and<B>(Option<B> b) => TwoOption(this, b);
}

// Replace with static extension methods on tuple where each param is an Option when static extension methods land.
class TwoOption<A, B> extends Tuple2<Option<A>, Option<B>> {

  TwoOption(Option<A> a, Option<B> b) : super(a, b);

  Option<Z> map<Z>(Z Function(A, B) map) => Option.map2(value1, value2, map);

  ThreeOption<A, B, Z> and<Z>(Option<Z> other) => ThreeOption(value1, value2, other);
}

class ThreeOption<A, B, C> extends Tuple3<Option<A>, Option<B>, Option<C>>{
  ThreeOption(Option<A> a, Option<B> b, Option<C> c): super(a, b, c);

  Option<Z> map<Z>(Z Function(A, B, C) map) => Option.map3(value1, value2, value3, map);

  FourOption<A, B, C, Z> and<Z>(Option<Z> other) => FourOption(value1, value2, value3, other);
}

class FourOption<A, B, C, D> extends Tuple4<Option<A>, Option<B>, Option<C>, Option<D>>{

  FourOption(Option<A> a, Option<B> b, Option<C> c, Option<D> d) : super(a, b, c, d);

  Option<Z> map<Z>(Z Function(A, B, C, D) map) => Option.map4(value1, value2, value3, value4, map);

  FiveOption<A, B, C, D, Z> and<Z>(Option<Z> other) => FiveOption(value1, value2, value3, value4, other);
}

class FiveOption<A, B, C, D, E> {
  final Option<A> a;
  final Option<B> b;
  final Option<C> c;
  final Option<D> d;
  final Option<E> e;

  const FiveOption(this.a, this.b, this.c, this.d, this.e);

  Option<Z> map<Z>(Z Function(A, B, C, D, E) map) => Option.map5(a, b, c, d, e, map);

  SixOption<A, B, C, D, E, Z> and<Z>(Option<Z> other) => SixOption(a, b, c, d, e, other);
}

class SixOption<A, B, C, D, E, F> {
  final Option<A> a;
  final Option<B> b;
  final Option<C> c;
  final Option<D> d;
  final Option<E> e;
  final Option<F> f;

  const SixOption(this.a, this.b, this.c, this.d, this.e, this.f);

  Option<Z> map<Z>(Z Function(A, B, C, D, E, F) map) => Option.map6(a, b, c, d, e, f, map);
}