import 'dart:convert';

import 'package:bird/smart_json.dart';
import 'package:meta/meta.dart';

class IsJSONSmartCodec<T> extends Codec<Object, T> {

  final T Function(Object) fromJSON;

  /// Set to true if you're using enums. They will be serialized to json with their .toString() method.
  final bool useToStringToJson;

  const IsJSONSmartCodec({
    @required this.fromJSON,
    this.useToStringToJson = false,
  });

  @override
  Converter<T, Object> get decoder =>
      AnonymousConverter<T, Object>((T j) => _dynamicToJson(j));

  @override
  Converter<Object, T> get encoder =>
      AnonymousConverter<Object, T>((Object f) => this.fromJSON(f));

  Object _dynamicToJson(Object j) {
    if (j is JSONSmartVariableInterface) {
      return j.to();
    } else if (j is SmartJSON) {
      return j.to();
    } else if (j is Iterable) {
      return j.map<dynamic>((dynamic a) => _dynamicToJson(a)).toList();
    } else if (j is Map) {
      return j.map<dynamic, dynamic>((dynamic a, dynamic b) =>
          MapEntry<dynamic, dynamic>(a, _dynamicToJson(b)));
    } else {
      if (useToStringToJson) {
        return j.toString();
      } else {
        return j;
      }
    }
  }
}

class PassThroughCodec<T> extends Codec<dynamic, T> {

  const PassThroughCodec();

  @override
  Converter<T, dynamic> get decoder =>
      AnonymousConverter<T, dynamic>((T j) => j);

  @override
  Converter<dynamic, T> get encoder =>
      AnonymousConverter<dynamic, T>((dynamic f) => f as T);
}

class ToStringCodec<T> extends Codec<dynamic, T> {

  final T Function(dynamic) fromJSON;

  const ToStringCodec({@required this.fromJSON});

  @override
  Converter<T, dynamic> get decoder =>
      AnonymousConverter<T, dynamic>((T j) => j.toString());

  @override
  Converter<dynamic, T> get encoder =>
      AnonymousConverter<dynamic, T>((dynamic f) => fromJSON(f));
}