import 'package:bird/smart_api.dart';
import 'package:dartz/dartz.dart' hide State;

const UnknownStorageError unknownStorageError =
    UnknownStorageError(None(), None());

UnknownStorageError unknownHintedStorageError(String description,
        {StackTrace stacktrace}) =>
    UnknownStorageError(optionOf(description), optionOf(stacktrace));

/// General unknown error.
///
/// For == and hashCode only the type counts.
class UnknownStorageError extends StorageErrors {
  /// Usually exception.toString().
  final Option<String> description;

  final Option<StackTrace> stacktrace;

  const UnknownStorageError(this.description, this.stacktrace);

  @override
  bool operator ==(Object other) =>
      identical(this, other) ||
      other is UnknownStorageError && runtimeType == other.runtimeType;

  @override
  int get hashCode => 0;
}

const DataNotFoundStorageError notFoundInStorageError =
    DataNotFoundStorageError();

class DataNotFoundStorageError extends StorageErrors {
  const DataNotFoundStorageError();

  @override
  bool operator ==(Object other) =>
      identical(this, other) ||
      other is DataNotFoundStorageError && runtimeType == other.runtimeType;

  @override
  int get hashCode => 0;
}

FailedParsingStorageError failedParsingError(String desc) =>
    FailedParsingStorageError(desc);

class FailedParsingStorageError extends StorageErrors {
  final String description;

  const FailedParsingStorageError(this.description);

  @override
  bool operator ==(Object other) =>
      identical(this, other) ||
      other is FailedParsingStorageError && runtimeType == other.runtimeType;

  @override
  int get hashCode => 0;
}
