/// This is meant to be used with [SocketEndpointImpl] &
/// [SocketChannelServiceEndpoint] and describes
/// when the endpoint should change into a connected state.
abstract class IsConnectedPolicy {}

/// With this policy the subject will not go into the connected
/// state on its own. It is the responsibility of a different
/// actor to turn the subject into a connected state.
///
/// In the case of socket connections and socket channel connections reducers
/// are supposed to execute the switch to a connected state.
class DontAutoConnectPolicy implements IsConnectedPolicy {
  const DontAutoConnectPolicy();
}

/// This policy indicates that a connection should become connected
/// immediately after initiation.
class ConnectImmediatelyPolicy implements IsConnectedPolicy {
  const ConnectImmediatelyPolicy();
}

/// This policy indicates that a connection should become connected
/// immediately after the first message was received
class IsConnectedOnFirstMessagePolicy implements IsConnectedPolicy {
  const IsConnectedOnFirstMessagePolicy();
}
