import 'dart:collection';

import 'package:bird/signal_undo_redo.dart';
import 'package:dartz/dartz.dart';
import 'package:meta/meta.dart';
import 'package:test/test.dart';

void main() {
  group("$UndoManager", () {
    /// Nothing to test
  });

  group("$UndoCommand", () {
    /// Nothing to test
  });

  group("$UndoManagerImpl", () {
    test("default limits", () {
      // ignore: close_sinks
      final UndoManagerImpl a = UndoManagerImpl();
      expect(a.undoLimit, 100);
      expect(a.redoLimit, 100);
    });

    final ac = SomeCommand("1");
    UndoCommandContainer<SomeCommand> a(bool e) =>
        UndoCommandContainer<SomeCommand>(ac,
            e ? DRUCommandType.execute : DRUCommandType.undo, DateTime.now());
    final bc = SomeCommand("2");
    UndoCommandContainer<SomeCommand> b(bool e) =>
        UndoCommandContainer<SomeCommand>(bc,
            e ? DRUCommandType.execute : DRUCommandType.undo, DateTime.now());
    final cc = SomeCommand("3");
    UndoCommandContainer<SomeCommand> c(bool e) =>
        UndoCommandContainer<SomeCommand>(cc,
            e ? DRUCommandType.execute : DRUCommandType.undo, DateTime.now());
    final empty = Queue<UndoCommandContainer<SomeCommand>>.from(
        <UndoCommandContainer<SomeCommand>>[]);

    group("push", () {
      /// TODO test redo and undo both execute correct methods
      test("adding elements executes that element immediately", () {
        final List<int> executed = [];
        final List<int> undid = [];
        // ignore: close_sinks
        final UndoManagerImpl<AnonymousUndoCommand> sut =
            UndoManagerImpl(undoLimit: 2, redoLimit: 2);

        sut.add(AnonymousUndoCommand(
          onExecute: () => executed.add(1),
          onUndo: () => undid.add(2),
        ));

        expect(executed, [1]);
        expect(undid, <int>[]);
      });
      test("adds elements and trims undo overflow", () {
        // ignore: close_sinks
        final UndoManagerImpl<SomeCommand> sut =
            UndoManagerImpl(undoLimit: 2, redoLimit: 2);
        expect(sut.redoList.value, empty);
        expect(sut.undoList.value, empty);
        sut.add(ac);
        expect(sut.redoList.value, empty);
        expect(
            sut.undoList.value,
            Queue<UndoCommandContainer<SomeCommand>>.from(
                <UndoCommandContainer<SomeCommand>>[a(true)]));
        sut.add(bc);
        sut.add(cc);
        expect(sut.redoList.value, empty);
        expect(
            sut.undoList.value,
            Queue<UndoCommandContainer<SomeCommand>>.from(
                <UndoCommandContainer<SomeCommand>>[b(true), c(true)]));
      });
      test("push clears redo list", () {
        // ignore: close_sinks
        final UndoManagerImpl<SomeCommand> sut =
            UndoManagerImpl(undoLimit: 2, redoLimit: 2);
        sut.add(ac);
        sut.add(bc);
        sut.undo();
        sut.undo();
        expect(
            sut.redoList.value,
            Queue<UndoCommandContainer<SomeCommand>>.from(
                <UndoCommandContainer<SomeCommand>>[b(false), a(false)]));
        expect(
            sut.undoList.value,
            Queue<UndoCommandContainer<SomeCommand>>.from(
                <UndoCommandContainer<SomeCommand>>[]));
        sut.add(cc);
        expect(
            sut.redoList.value,
            Queue<UndoCommandContainer<SomeCommand>>.from(
                <UndoCommandContainer<SomeCommand>>[]));
        expect(
            sut.undoList.value,
            Queue<UndoCommandContainer<SomeCommand>>.from(
                <UndoCommandContainer<SomeCommand>>[c(true)]));
      });
    });

    group("undo", () {
      test("adds elements to redo list", () {
        // ignore: close_sinks
        final UndoManagerImpl<SomeCommand> sut =
            UndoManagerImpl(undoLimit: 2, redoLimit: 2);
        sut.add(ac);
        sut.add(bc);
        sut.undo();
        expect(
            sut.redoList.value,
            Queue<UndoCommandContainer<SomeCommand>>.from(
                <UndoCommandContainer<SomeCommand>>[b(false)]));
        expect(
            sut.undoList.value,
            Queue<UndoCommandContainer<SomeCommand>>.from(
                <UndoCommandContainer<SomeCommand>>[a(true)]));
      });

      test("trims redo overflow", () {
        // ignore: close_sinks
        final UndoManagerImpl<SomeCommand> sut =
            UndoManagerImpl(undoLimit: 3, redoLimit: 2);
        sut.add(ac);
        sut.add(bc);
        sut.add(cc);
        sut.undo();
        sut.undo();
        sut.undo();
        expect(
            sut.redoList.value,
            Queue<UndoCommandContainer<SomeCommand>>.from(
                <UndoCommandContainer<SomeCommand>>[c(false), b(false)]));
        expect(
            sut.undoList.value,
            Queue<UndoCommandContainer<SomeCommand>>.from(
                <UndoCommandContainer<SomeCommand>>[]));
      });
    });

    group("redo", () {
      test("adds elements back to undo", () {
        // ignore: close_sinks
        final UndoManagerImpl<SomeCommand> sut =
            UndoManagerImpl(undoLimit: 2, redoLimit: 2);
        sut.add(ac);
        sut.add(bc);
        expect(
            sut.redoList.value,
            Queue<UndoCommandContainer<SomeCommand>>.from(
                <UndoCommandContainer<SomeCommand>>[]));
        expect(
            sut.undoList.value,
            Queue<UndoCommandContainer<SomeCommand>>.from(
                <UndoCommandContainer<SomeCommand>>[a(true), b(true)]));
        sut.undo();
        sut.undo();
        expect(
            sut.redoList.value,
            Queue<UndoCommandContainer<SomeCommand>>.from(
                <UndoCommandContainer<SomeCommand>>[b(false), a(false)]));
        expect(
            sut.undoList.value,
            Queue<UndoCommandContainer<SomeCommand>>.from(
                <UndoCommandContainer<SomeCommand>>[]));
        sut.redo();
        expect(
            sut.redoList.value,
            Queue<UndoCommandContainer<SomeCommand>>.from(
                <UndoCommandContainer<SomeCommand>>[b(false)]));
        expect(
            sut.undoList.value,
            Queue<UndoCommandContainer<SomeCommand>>.from(
                <UndoCommandContainer<SomeCommand>>[a(true)]));
        sut.redo();
        expect(
            sut.redoList.value,
            Queue<UndoCommandContainer<SomeCommand>>.from(
                <UndoCommandContainer<SomeCommand>>[]));
        expect(
            sut.undoList.value,
            Queue<UndoCommandContainer<SomeCommand>>.from(
                <UndoCommandContainer<SomeCommand>>[a(true), b(true)]));
      });

      test("trims undo overflow", () {
        // ignore: close_sinks
        final UndoManagerImpl<SomeCommand> sut =
            UndoManagerImpl(undoLimit: 2, redoLimit: 3);
        sut.redoList.add(Queue<UndoCommandContainer<SomeCommand>>.from(
            <UndoCommandContainer<SomeCommand>>[c(false), b(false), a(false)]));
        sut.redo();
        sut.redo();
        expect(
            sut.redoList.value,
            Queue<UndoCommandContainer<SomeCommand>>.from(
                <UndoCommandContainer<SomeCommand>>[c(false)]));
        expect(
            sut.undoList.value,
            Queue<UndoCommandContainer<SomeCommand>>.from(
                <UndoCommandContainer<SomeCommand>>[a(true), b(true)]));
        sut.redo();
        expect(
            sut.redoList.value,
            Queue<UndoCommandContainer<SomeCommand>>.from(
                <UndoCommandContainer<SomeCommand>>[]));
        expect(
            sut.undoList.value,
            Queue<UndoCommandContainer<SomeCommand>>.from(
                <UndoCommandContainer<SomeCommand>>[b(true), c(true)]));
      });
    });

    test("canRedo", () {
      // ignore: close_sinks
      final UndoManagerImpl a = UndoManagerImpl();
      expect(a.canRedo.value, false);
      a.redoList.add(Queue.from(<UndoCommandContainer>[
        UndoCommandContainer(
            SomeCommand("-"), DRUCommandType.execute, DateTime.now())
      ]));
      expect(a.canRedo.value, true);
    });

    test("canUndo", () {
      // ignore: close_sinks
      final UndoManagerImpl a = UndoManagerImpl();
      expect(a.canUndo.value, false);
      a.undoList.add(Queue.from(<UndoCommandContainer>[
        UndoCommandContainer(
            SomeCommand("-"), DRUCommandType.execute, DateTime.now())
      ]));
      expect(a.canUndo.value, true);
    });

    group("commands", () {
      test("going through a scenario", () {
        final UndoManagerImpl<SomeCommand> sut =
            UndoManagerImpl(undoLimit: 2, redoLimit: 2);
        expect(
            sut.commands.asStream(),
            emitsInOrder(<dynamic>[
              const None<UndoCommandContainer>(),
//                    Some(a(true)),
//                    Some(b(true)),
//                    Some(b(false)),
//                    Some(b(true)),
//                    Some(b(false)),
//                    Some(c(true)),
//                    Some(c(false)),
//                    Some(a(false)),
//                    Some(a(true)),
//                    Some(c(true)),
//                    Some(c(false)),
//                    emitsDone,
            ]));

        sut.add(ac);
        sut.add(bc);
        sut.undo();
        sut.redo();
        sut.undo();
        sut.add(cc);
        sut.undo();
        sut.undo();
        sut.redo();
        sut.redo();

        expect(
            sut.redoList.value,
            Queue<UndoCommandContainer<SomeCommand>>.from(
                <UndoCommandContainer<SomeCommand>>[]));
        expect(
            sut.undoList.value,
            Queue<UndoCommandContainer<SomeCommand>>.from(
                <UndoCommandContainer<SomeCommand>>[a(true), c(true)]));

        sut.undo();

        expect(
            sut.redoList.value,
            Queue<UndoCommandContainer<SomeCommand>>.from(
                <UndoCommandContainer<SomeCommand>>[c(false)]));
        expect(
            sut.undoList.value,
            Queue<UndoCommandContainer<SomeCommand>>.from(
                <UndoCommandContainer<SomeCommand>>[a(true)]));

        sut.close();
      });
    });

    test("dispose", () async {
      final UndoManagerImpl a = UndoManagerImpl();
      expect(a.undoList.isClosed, false);
      expect(a.redoList.isClosed, false);
      expect(a.pcommands.isClosed, false);
      a.close();
      expect(a.undoList.isClosed, true);
      expect(a.redoList.isClosed, true);
      expect(a.pcommands.isClosed, true);
    });
  });

  group("$UndoCommandContainer", () {
    test("==", () {
      expect(
          UndoCommandContainer(
              SomeCommand("1"), DRUCommandType.execute, DateTime.now()),
          UndoCommandContainer(
              SomeCommand("1"), DRUCommandType.execute, DateTime.now()));
    });

    test("!=", () {
      expect(
          UndoCommandContainer(
              SomeCommand("1"), DRUCommandType.undo, DateTime.now()),
          isNot(UndoCommandContainer(
              SomeCommand("1"), DRUCommandType.execute, DateTime.now())));

      expect(
          UndoCommandContainer(
              SomeCommand("2"), DRUCommandType.execute, DateTime.now()),
          isNot(UndoCommandContainer(
              SomeCommand("1"), DRUCommandType.execute, DateTime.now())));
    });

    test("hashCode", () {
      expect(
          UndoCommandContainer(
                  SomeCommand("1"), DRUCommandType.execute, DateTime.now())
              .hashCode,
          UndoCommandContainer(
                  SomeCommand("1"), DRUCommandType.execute, DateTime.now())
              .hashCode);
    });
  });
}

class SomeCommand implements UndoCommand {
  final String t;

  SomeCommand(this.t);

  @override
  void execute() {}

  @override
  void undo() {}

  @override
  bool operator ==(Object other) =>
      identical(this, other) ||
      other is SomeCommand && runtimeType == other.runtimeType && t == other.t;

  @override
  int get hashCode => t.hashCode;

  @override
  String toString() => 'SomeCommand{t: $t}';
}

class AnonymousUndoCommand implements UndoCommand {
  final void Function() onExecute;
  final void Function() onUndo;

  const AnonymousUndoCommand({
    @required this.onExecute,
    @required this.onUndo,
  });

  @override
  void execute() => onExecute();

  @override
  void undo() => onUndo();
}
