import 'package:bird/smart_api.dart';
import 'package:dartz/dartz.dart';
import 'package:test/test.dart';

void main() {
    group("$APIState", () {
        test("missing hanndle implementation", () {
            expect(() =>
                _TestAPILayer6State().visit(
                    initialized: (_) => false,
                    connected: (_) => false,
                    loading: (_) => false,
                    disconnected: (_) => false,
                ), throwsA(const TypeMatcher<UnimplementedError>()));
        });
    });

    test("apiInitLoadConnect", () {
        expect(apiInitLoadConnect, [apiInitialized, apiLoading, apiConnected]);
    });

    group("$APIStateInitialized", () {
        test("handle", () {
            expect(const APIStateInitialized().visit(
                initialized: (_) => true,
                connected: (_) => false,
                loading: (_) => false,
                disconnected: (_) => false,
            ), true);
        });

        test("==", () {
            // ignore: prefer_const_constructors
            expect(const APIStateInitialized(), APIStateInitialized());
        });

        test("hashCode", () {
            expect(const APIStateInitialized().hashCode, const APIStateInitialized().hashCode);
        });

        test("apiInitialized", () {
            expect(apiInitialized, const APIStateInitialized());
        });
    });

    group("$APIStateConnected", () {
        test("handle", () {
            expect(const APIStateConnected().visit(
                initialized: (_) => false,
                connected: (_) => true,
                loading: (_) => false,
                disconnected: (_) => false,
            ), true);
        });
        test("==", () {
            expect(const APIStateConnected(), const APIStateConnected());
        });

        test("!=", () {
            expect(const APIStateConnected(), isNot(const APIStateDisconnected(Some(NoInternetAPIError()))));
        });

        test("hashCode", () {
            expect(const APIStateConnected().hashCode, const APIStateConnected().hashCode);
        });

        test("apiConnected", () {
            expect(apiConnected, const APIStateConnected());
        });
    });

    group("$APIStateLoading", () {
        test("handle", () {
            expect(const APIStateLoading().visit(
                initialized: (_) => false,
                connected: (_) => false,
                loading: (_) => true,
                disconnected: (_) => false,
            ), true);
        });

        test("==", () {
            expect(const APIStateLoading(), const APIStateLoading());
        });

        test("!=", () {
            expect(const APIStateLoading(), isNot(const APIStateConnected()));
        });

        test("hashCode", () {
            expect(const APIStateLoading().hashCode, const APIStateLoading().hashCode);
        });

        test("apiLoading", () {
            expect(apiLoading, const APIStateLoading());
        });
    });

    group("$APIStateDisconnected", () {
        test("handle", () {
            expect(const APIStateDisconnected(Some(NoInternetAPIError())).visit(
                initialized: (_) => false,
                connected: (_) => false,
                loading: (_) => false,
                disconnected: (_) => true,
            ), true);
        });

        test("==", () {
            expect(const APIStateDisconnected(Some(NoInternetAPIError())), const APIStateDisconnected(Some(NoInternetAPIError())));
        });

        test("!=", () {
            expect(const APIStateDisconnected(Some(NoInternetAPIError())), isNot(const APIStateLoading()));
            expect(const APIStateDisconnected(Some(NoInternetAPIError())), isNot(APIStateDisconnected(Some(statusError(123)))));
        });

        test("hashCode", () {
            expect(
                const APIStateDisconnected(Some(NoInternetAPIError())).hashCode, const APIStateDisconnected(Some(NoInternetAPIError())).hashCode);
        });

        test("toString", () {
            expect(APIStateDisconnected(Some(statusError(123))).toString(), '$APIStateDisconnected{error: ${Some(statusError(123))}}');
        });

        test("apiDisconnected", () {
            expect(apiDisconnected(const NoInternetAPIError()), const APIStateDisconnected(Some(NoInternetAPIError())));
        });
    });
}

class _TestAPILayer6State extends APIState {
}